;
;   TIMER
;

.nolist
.include "1200def.inc"
.include "avr.inc"
.list


.cseg
        ; Vectors
        rjmp    main
        rjmp    0       ; INT0 not used
        rjmp    timer_intr
        rjmp    0       ; Analog comparator not used


main:
        clr     r16
        mov     r7,r16          ; Timer mode = STOP
        out     GIMSK,r16       ; Disable INT0

        ; Initialize Timer/Counter

        ldi     r16,0b00000011  ; CK / 64
        out     TCCR0,r16

        ldi     r16,255 - 183
        out     TCNT0,r16

        ldi     r16,0b00000010  ; Enable T/C int
        out     TIMSK,r16

        ; Initialize I/O port

        clr     r16		; All segments OFF
        out     PORTB,r16
        ldi     r16,0b11111111	; Output mode
        out     DDRB,r16

        ldi     r16,0b01100000	; Pull-up KEY port
        out     PORTD,r16
        ldi     r16,0b00011111	; [65]:In(KEY) [4]:Out(BZ) [3-0]:Out(LED COM)
        out     DDRD,r16

        clr     r24
        ldi     r25,3
        clr     r7
        clr     r4

        clr     r2

        sei

        rjmp    show_min

        ; Event loop
loop:
        tst     r2
        breq    loop            ; No key

        tst     r7
        brne    loop0


	; Mode 0 (Settings)
	mov	r16,r2
	clr	r2
	cpi	r16,0x11
	breq	inc_time
	cpi	r16,0x31
	breq	inc_time
	cpi	r16,0x12
	breq	dec_time
	cpi	r16,0x32
	breq	dec_time
	cpi	r16,0x23
	breq	start
	rjmp	loop

inc_time:
        inc     r25
        cpi     r25,10
        brne    it0

        clr     r25
        inc     r24
        cpi     r24,10
        brne    it0

        ldi     r24,9
        ldi     r25,9

it0:
        rjmp    show_min

dec_time:
        dec     r25
        cpi     r25,255
        brne    dt0

        ldi     r25,9
        dec     r24
        cpi     r24,255
        brne    dt0

        clr     r24
        clr     r25

dt0:
        tst     r24
        brne    dt1
        tst     r25
        brne    dt1

        inc     r25             ; Set to 1min when 0min

dt1:
        rjmp    show_min


start:
        ldi     r16,255
        mov     r6,r16
        mov     r8,r24
        mov     r9,r25
        clr     r10
        clr     r11
        inc     r7              ; Start timer
        rjmp    loop


loop0:
        mov     r23,r7
        dec     r23
        brne    loop1

	; Mode 1 (Counting)
	mov	r16,r2
	clr	r2
	cpi	r16,0x23
	breq	stop
	rjmp	loop

stop:
        clr     r7              ; Stop timer
        rjmp    show_min


loop1:
        dec     r23
        brne    loop2

	; Mode 2 (Ringing)
	mov	r16,r2
	clr	r2
	cpi	r16,0x23
	breq	stop
	rjmp	loop

loop2:
	; Must not happen
	clr	r2
	rjmp	loop


show_min:
        mov     r12,r24
        mov     r13,r25
        ldi     r16,10
        mov     r14,r16
        mov     r15,r16
        tst     r12
        brne    sm0
        mov     r12,r16
sm0:
        rjmp    loop

timer_intr:
        inc     r5
        ldi     r31,0b00000011
        and     r31,r5
        brne    ti1

        ; Decrement time if timer running
        ldi     r31,1
        cp      r7,r31
        brne    ti1

        ldi     r30,1
        sub     r6,r30
        brcc    ti0

        sub     r11,r30         ; Decrement 1sec
        brcc    ti0
        ldi     r31,9
        mov     r11,r31

        sub     r10,r30         ; Decrement 10sec
        brcc    ti0
        ldi     r31,5
        mov     r10,r31

        sub     r9,r30          ; Decrement 1min
        brcc    ti0
        ldi     r31,9
        mov     r9,r31

        sub     r8,r30          ; Decrement 10min
        brcc    ti0

        inc     r7              ; Stop timer
        rjmp    ti1

ti0:
        mov     r31,r8
        or      r31,r9
        or      r31,r10
        or      r31,r11
        brne    ti05

        inc     r7              ; Time up

ti05:
        ; Update VRAM
        mov     r12,r8
        mov     r13,r9
        mov     r14,r10
        mov     r15,r11
        tst     r12
        brne    ti06
        ldi     r31,10
        mov     r12,r31

ti06:
        bst     r6,7
        brtc    ti1

        ldi     r31,0b10000000  ; Blinkning decimal point
        or      r13,r31

ti1:
        ; Check Key

        in      r30,PIND
        lsr     r30
        lsr     r30
        lsr     r30
        lsr     r30
        lsr     r30
        com     r30
        andi    r30,0b00000011	; Got key code
	tst	r4
	breq	ti2		; if(last)

	cp	r4,r30
	brne	ti15		; if(key == last) (

	inc	r3		;   count++;
	ldi	r31,4
	cp	r3,r31
	brne	ti12		;   if(count == 4) (

	ldi	r31,0x10
	or	r31,r4
	mov	r2,r31		;     KeyPress(last);

	clr	r1

	rjmp	ti3
ti12:	 
	tst	r3
	brne	ti3		;   } else if(count > 255) (

	ldi	r31,0x30
	or	r31,r4
	mov	r2,r31		;     KeyRepeat(last);

	ldi	r31,192
	mov	r3,r31		;     count = 192;

	inc	r1
	tst	r1
	brne	ti3
	dec	r1

	rjmp	ti3
ti15:				; } else (

	ldi	r31,4
	cp	r3,r31
	brcs	ti17		;   if(count >= 4) (

	ldi	r31,0x20
	or	r31,r4
	mov	r2,r31		;     KeyRelease(last);

ti17:				;   }

	mov	r4,r30		;   last = key;
	clr	r3		;   count = 0;

	rjmp	ti3		; }
ti2:
	tst	r30
	breq	ti3

	mov	r4,r30		; last = key;
	clr	r3		; count = 0;

ti3:
        ; Scan LED display

        in      r31,PORTD
        andi    r31,0b11110000  ; Turn off all digit driver
        out     PORTD,r31

        mov     r30,r5
        andi    r30,0b00000011
        addi    r30,12
        ld      r31,Z
        mov     r29,r31
        andi    r31,0b00001111  ; Mask attribute
        ldi     r30,0b00111111  ; 0
        tst     r31
        breq    ti4
        ldi     r30,0b00000110  ; 1
        dec     r31
        breq    ti4
        ldi     r30,0b01011011  ; 2
        dec     r31
        breq    ti4
        ldi     r30,0b01001111  ; 3
        dec     r31
        breq    ti4
        ldi     r30,0b01100110  ; 4
        dec     r31
        breq    ti4
        ldi     r30,0b01101101  ; 5
        dec     r31
        breq    ti4
        ldi     r30,0b01111101  ; 6
        dec     r31
        breq    ti4
        ldi     r30,0b00000111  ; 7
        dec     r31
        breq    ti4
        ldi     r30,0b01111111  ; 8
        dec     r31
        breq    ti4
        ldi     r30,0b01101111  ; 9
        dec     r31
        breq    ti4
        clr     r30             ; Blank
ti4:
        bst     r29,7
        brtc    ti41

        ori     r30,0b10000000  ; Turn on decimal point
ti41:
        bst     r29,6
        brtc    ti45

        bst     r5,6            ; Blinking interval
        brts    ti45

        clr     r30             ; Blank
ti45:
        out     PORTB,r30       ; Set segment driver

        ldi     r31,0b00000001
        mov     r30,r5
        andi    r30,0b00000011
ti5:
        tst     r30
        breq    ti6
        dec     r30
        lsl     r31
        rjmp    ti5
ti6:
        in      r30,PORTD
        or      r31,r30
        out     PORTD,r31       ; Set digit driver

        ; Bell when timed out

        ldi     r31,2
        cp      r7,r31
        brne    ti9
        ldi     r31,0b01000001
        and     r31,r5
        brne    ti7
        sbi     PORTD,4
        rjmp    ti8
ti7:
        cbi     PORTD,4
ti8:
        
ti9:
        ldi     r31,255 - 183
        out     TCNT0,r31

        reti
